@@ -625,10 +625,6 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
         # Decide what to execute.
         if source is None:
             source = self.input_buffer
-            if not hidden:
-                # A newline is appended later, but it should be considered part
-                # of the input buffer.
-                source += '\n'
         elif not hidden:
             self.input_buffer = source
 
@@ -1908,7 +1904,7 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
     def _indent(self, dedent=True):
         """ Indent/Dedent current line or current text selection.
         """
-        num_newlines = self._get_cursor().selectedText().count("\u2029")
+        num_newlines = self._get_cursor().selectedText().count(u"\u2029")
         save_cur = self._get_cursor()
         cur = self._get_cursor()
 
@@ -2024,6 +2020,13 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
         if buffer_size > 0:
             text = self._get_last_lines(text, buffer_size)
 
+        if self._executing:
+            viewport = self._control.viewport()
+            end_scroll_pos = self._control.cursorForPosition(
+                QtCore.QPoint(viewport.width()-1, viewport.height()-1)
+                ).position()
+            end_doc_pos = self._get_end_pos()
+
         cursor.beginEditBlock()
         if self.ansi_codes:
             for substring in self._ansi_processor.split_string(text):
@@ -2084,6 +2087,9 @@ class ConsoleWidget(MetaQObjectHasTraits('NewBase', (LoggingConfigurable, superQ
             cursor.insertText(text)
         cursor.endEditBlock()
 
+        if self._executing and end_doc_pos - end_scroll_pos <= 1:
+            self._control.moveCursor(QtGui.QTextCursor.End)
+
     def _insert_plain_text_into_buffer(self, cursor, text):
         """ Inserts text into the input buffer using the specified cursor (which
             must be in the input buffer), ensuring that continuation prompts are
